@@ -38,23 +38,29 @@ typedef struct {
 	 void (*set_bounds) (BonoboCanvasComponent *component,
 			     Bonobo_Canvas_DRect *bbox,
 			     CORBA_Environment *ev);
+
+	 gboolean (*event) (BonoboCanvasComponent *component, GdkEvent *event);
 } BonoboCanvasComponentClass;
 
 GtkType                 bonobo_canvas_component_get_type         (void);
 Bonobo_Canvas_Component bonobo_canvas_component_object_create    (BonoboObject                *object);
-void                    bonobo_canvas_component_set_proxy        (BonoboCanvasComponent       *comp,
-								  Bonobo_Canvas_ComponentProxy proxy);
 BonoboCanvasComponent  *bonobo_canvas_component_construct        (BonoboCanvasComponent       *comp,
 								  Bonobo_Canvas_Component      corba_canvas_comp,
 								  GnomeCanvasItem             *item);
 BonoboCanvasComponent  *bonobo_canvas_component_new              (GnomeCanvasItem             *item);
 GnomeCanvasItem        *bonobo_canvas_component_get_item         (BonoboCanvasComponent       *comp);
-void                    bonobo_canvas_component_set_ui_component (BonoboCanvasComponent       *comp,
-								  Bonobo_UIComponent           ui_component);
-Bonobo_UIComponent      bonobo_canvas_component_get_ui_component (BonoboCanvasComponent       *comp);
+void		        bonobo_canvas_component_grab		 (BonoboCanvasComponent       *comp, guint mask, GdkCursor *cursor, guint32 time);
+void		        bonobo_canvas_component_ungrab		 (BonoboCanvasComponent       *comp, guint32 time);
+Bonobo_UIContainer      bonobo_canvas_component_get_ui_container (BonoboCanvasComponent       *comp);
 					  
 POA_Bonobo_Canvas_Component__epv *bonobo_canvas_component_get_epv  (void);
 
+/* This is a helper function for creating a canvas with the root item replaced
+ * by a proxy to the client side proxy.
+ */
+GnomeCanvas *bonobo_canvas_new (gboolean is_aa,
+				Bonobo_Canvas_ComponentProxy proxy);
+
 END_GNOME_DECLS
 
 #endif /* */
