@@ -44,8 +44,7 @@ impl_Bonobo_ClientSite_getContainer (PortableServer_Servant servant, CORBA_Envir
 	Bonobo_ItemContainer  corba_object;
 	BonoboClientSite     *client_site = BONOBO_CLIENT_SITE (object);
 
-	corba_object = bonobo_object_corba_objref (
-		BONOBO_OBJECT (client_site->container));
+	corba_object = BONOBO_OBJREF (client_site->container);
 
 	return bonobo_object_dup_ref (corba_object, ev);
 }
@@ -98,12 +97,9 @@ bonobo_client_site_destroy (GtkObject *object)
 	 */
 	while (client_site->canvas_items) {
 		BonoboCanvasItem *item = BONOBO_CANVAS_ITEM (client_site->canvas_items->data);
-
-		bonobo_object_unref (BONOBO_OBJECT (item));
+		gtk_object_unref (GTK_OBJECT (item));
 	}
 
-	bonobo_item_container_remove (client_site->container, BONOBO_OBJECT (object));
-
 	if (client_site->bound_embeddable) {
 		bonobo_object_unref (BONOBO_OBJECT (client_site->bound_embeddable));
 		client_site->bound_embeddable = NULL;
@@ -238,7 +234,6 @@ bonobo_client_site_construct (BonoboClientSite    *client_site,
 	bonobo_object_construct (BONOBO_OBJECT (client_site), corba_client_site);
 	
 	BONOBO_CLIENT_SITE (client_site)->container = container;
-	bonobo_item_container_add (container, BONOBO_OBJECT (client_site));
 
 	return client_site;
 }
@@ -341,13 +336,10 @@ bonobo_client_site_bind_embeddable (BonoboClientSite   *client_site,
 	CORBA_exception_init (&ev);
 
 	/* The QI adds a ref */
-	Bonobo_Unknown_unref (bonobo_object_corba_objref (
-		BONOBO_OBJECT (object)), &ev);
+	Bonobo_Unknown_unref (BONOBO_OBJREF (object), &ev);
 
-	Bonobo_Embeddable_setClientSite (
-		embeddable_object, 
-		bonobo_object_corba_objref (BONOBO_OBJECT (client_site)),
-		&ev);
+	Bonobo_Embeddable_setClientSite (embeddable_object, 
+					 BONOBO_OBJREF (client_site), &ev);
 		
 	if (BONOBO_EX (&ev)) {
 		bonobo_object_check_env (BONOBO_OBJECT (object),
@@ -454,12 +446,10 @@ bonobo_client_site_new_view_full (BonoboClientSite  *client_site,
 	/*
 	 * 2. Now, create the view.
 	 */
-	server_object = bonobo_object_corba_objref (BONOBO_OBJECT (client_site->bound_embeddable));
+	server_object = BONOBO_OBJREF (client_site->bound_embeddable);
 	CORBA_exception_init (&ev);
- 	view = Bonobo_Embeddable_createView (
-		server_object,
-		bonobo_object_corba_objref (BONOBO_OBJECT (view_frame)),
-		&ev);
+ 	view = Bonobo_Embeddable_createView (server_object, 
+					     BONOBO_OBJREF (view_frame), &ev);
 
 	if (BONOBO_EX (&ev)) {
 		bonobo_object_check_env (
@@ -517,15 +507,17 @@ canvas_item_destroyed (GnomeCanvasItem *item, BonoboClientSite *client_site)
 /**
  * bonobo_client_site_new_item:
  * @client_site: The client site that contains a remote Embeddable object
+ * @uic: The UI container for the item.
  * @group: The Canvas group that will be the parent for the new item.
  *
  * Returns: A GnomeCanvasItem that wraps the remote Canvas Item.
  */
 GnomeCanvasItem *
-bonobo_client_site_new_item (BonoboClientSite *client_site, GnomeCanvasGroup *group)
+bonobo_client_site_new_item (BonoboClientSite *client_site, Bonobo_UIContainer uic,
+			     GnomeCanvasGroup *group)
 {
-	BonoboObjectClient *server_object;
 	GnomeCanvasItem *item;
+	Bonobo_Embeddable corba_emb;
 		
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
@@ -533,12 +525,14 @@ bonobo_client_site_new_item (BonoboClientSite *client_site, GnomeCanvasGroup *gr
 	g_return_val_if_fail (group != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS_GROUP (group), NULL);
 
-	server_object = client_site->bound_embeddable;
+	corba_emb = BONOBO_OBJREF (client_site->bound_embeddable);
 
-	item = bonobo_canvas_item_new (group, server_object);
+	item = gnome_canvas_item_new (group, bonobo_canvas_item_get_type (),
+				      "corba_ui_container", uic, 
+				      "corba_embeddable", corba_emb, NULL); 
 
 	/*
-	 * 5. Add this new view frame to the list of ViewFrames for
+	 * 5. Add this new item to the list of CanvasItems for
 	 * this embedded component.
 	 */
 	client_site->canvas_items = g_list_prepend (client_site->canvas_items, item);
