@@ -11,6 +11,7 @@
  */
 #include <config.h>
 #include <gtk/gtksignal.h>
+#include <bonobo/bonobo-listener.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-event-source.h>
 #include <bonobo/bonobo-running-context.h>
@@ -395,3 +396,118 @@ bonobo_event_source_ignore_listeners (BonoboEventSource *event_source)
 
 	event_source->priv->ignore = TRUE;
 }
+
+void
+bonobo_event_source_client_remove_listener (Bonobo_Unknown  object,
+					    Bonobo_EventSource_ListenerId id,
+					    CORBA_Environment *opt_ev)
+{
+	CORBA_Environment ev, *my_ev;
+	Bonobo_Unknown es;
+
+	g_return_if_fail (object != CORBA_OBJECT_NIL);
+	g_return_if_fail (id != 0);
+
+	if (!opt_ev) {
+		CORBA_exception_init (&ev);
+		my_ev = &ev;
+	} else
+		my_ev = opt_ev;
+
+	if (CORBA_Object_is_a (object, "IDL:Bonobo/Property:1.0", my_ev)) {
+
+		Bonobo_Property_removeListener (object, id, my_ev);
+
+	} else { 
+
+		es = Bonobo_Unknown_queryInterface (object, 
+		       "IDL:Bonobo/EventSource:1.0", my_ev);
+
+		if (BONOBO_EX(my_ev) || !es)
+			goto remove_listener_end;
+
+		Bonobo_EventSource_removeListener (es, id, my_ev);
+
+		Bonobo_Unknown_unref (es, my_ev);
+	}
+
+ remove_listener_end:
+
+	if (!opt_ev) {
+		if (BONOBO_EX (my_ev))
+			g_warning ("remove_listener failed '%s'",
+				   bonobo_exception_get_text (my_ev));
+		CORBA_exception_free (&ev);
+	}
+}
+
+Bonobo_EventSource_ListenerId
+bonobo_event_source_client_add_listener (Bonobo_Unknown object,
+					 BonoboListenerCallbackFn event_callback,
+					 const char *opt_mask,
+					 CORBA_Environment *opt_ev,
+					 gpointer user_data)
+{
+	CORBA_Environment ev, *my_ev;
+	BonoboListener *listener = NULL;
+	Bonobo_Listener corba_listener;
+	Bonobo_EventSource_ListenerId id = 0;
+	Bonobo_Unknown es;
+
+	g_return_val_if_fail (object != CORBA_OBJECT_NIL, 0);
+	g_return_val_if_fail (event_callback != NULL, 0);
+	
+	if (!opt_ev) {
+		CORBA_exception_init (&ev);
+		my_ev = &ev;
+	} else
+		my_ev = opt_ev;
+
+	if (CORBA_Object_is_a (object, "IDL:Bonobo/Property:1.0", my_ev)) {
+		if (!(listener = bonobo_listener_new (event_callback, 
+						      user_data)))
+			goto add_listener_end;
+	      
+		corba_listener = BONOBO_OBJREF (listener);
+
+		id = Bonobo_Property_addListener (object, corba_listener, 
+						  my_ev);
+
+		bonobo_object_unref (BONOBO_OBJECT (listener));
+
+	} else {
+		es = Bonobo_Unknown_queryInterface (object, 
+		        "IDL:Bonobo/EventSource:1.0", my_ev);
+
+		if (BONOBO_EX(my_ev) || !es)
+			goto add_listener_end;
+
+		if (!(listener = bonobo_listener_new (event_callback, 
+						      user_data)))
+			goto add_listener_end;
+
+		corba_listener = BONOBO_OBJREF (listener);
+	
+		if (opt_mask)
+			id = Bonobo_EventSource_addListenerWithMask (es, 
+			        corba_listener, opt_mask, my_ev);
+		else 
+			id = Bonobo_EventSource_addListener (es, 
+				corba_listener, my_ev);
+
+		bonobo_object_unref (BONOBO_OBJECT (listener));
+		Bonobo_Unknown_unref (es, my_ev);
+	}
+
+ add_listener_end:
+
+	if (!opt_ev) {
+		if (BONOBO_EX (my_ev))
+			g_warning ("add_listener failed '%s'",
+				   bonobo_exception_get_text (my_ev));
+		CORBA_exception_free (&ev);
+	}
+
+	return id;
+} 
+
