@@ -20,20 +20,21 @@
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-generic-factory.h>
+#include <bonobo/bonobo-running-context.h>
 #include <liboaf/liboaf.h>
 
 POA_GNOME_ObjectFactory__vepv bonobo_generic_factory_vepv;
 
-static BonoboObjectClass *bonobo_generic_factory_parent_class;
+static BonoboObjectClass *bonobo_generic_factory_parent_class = NULL;
 
 static CORBA_boolean
 impl_Bonobo_ObjectFactory_manufactures (PortableServer_Servant  servant,
-					 const CORBA_char       *obj_goad_id,
+					 const CORBA_char       *obj_oaf_iid,
 					 CORBA_Environment      *ev)
 {
 	BonoboGenericFactory *factory = BONOBO_GENERIC_FACTORY (bonobo_object_from_servant (servant));
 
-	if (! strcmp (obj_goad_id, factory->goad_id))
+	if (! strcmp (obj_oaf_iid, factory->oaf_iid))
 		return CORBA_TRUE;
 
 	return CORBA_FALSE;
@@ -41,7 +42,7 @@ impl_Bonobo_ObjectFactory_manufactures (PortableServer_Servant  servant,
 
 static CORBA_Object
 impl_Bonobo_ObjectFactory_create_object (PortableServer_Servant   servant,
-					  const CORBA_char        *obj_goad_id,
+					  const CORBA_char        *obj_oaf_iid,
 					  const GNOME_stringlist *params,
 					  CORBA_Environment       *ev)
 {
@@ -52,7 +53,7 @@ impl_Bonobo_ObjectFactory_create_object (PortableServer_Servant   servant,
 	factory = BONOBO_GENERIC_FACTORY (bonobo_object_from_servant (servant));
 
 	class = BONOBO_GENERIC_FACTORY_CLASS (GTK_OBJECT (factory)->klass);
-	object = (*class->new_generic) (factory, obj_goad_id);
+	object = (*class->new_generic) (factory, obj_oaf_iid);
 
 	if (!object)
 		return CORBA_OBJECT_NIL;
@@ -60,15 +61,15 @@ impl_Bonobo_ObjectFactory_create_object (PortableServer_Servant   servant,
 	return CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (object)), ev);
 }
 
-static CORBA_Object
-create_bonobo_generic_factory (BonoboObject *object)
+CORBA_Object
+bonobo_generic_factory_corba_object_create (BonoboObject *object)
 {
 	POA_GNOME_ObjectFactory *servant;
 	CORBA_Environment ev;
 
 	CORBA_exception_init (&ev);
 
-	servant = (POA_GNOME_ObjectFactory *)g_new0 (BonoboObjectServant, 1);
+	servant = (POA_GNOME_ObjectFactory *) g_new0 (BonoboObjectServant, 1);
 	servant->vepv = &bonobo_generic_factory_vepv;
 
 	POA_GNOME_ObjectFactory__init ((PortableServer_Servant) servant, &ev);
@@ -84,13 +85,13 @@ create_bonobo_generic_factory (BonoboObject *object)
 
 /**
  * bonobo_generic_factory_construct:
- * @goad_id: The GOAD id that the new factory will implement.
+ * @oaf_iid: The GOAD id that the new factory will implement.
  * @c_factory: The object to be initialized.
  * @corba_factory: The CORBA object which supports the
  * Bonobo::GenericFactory interface and which will be used to
  * construct this BonoboGenericFactory Gtk object.
  * @factory: A callback which is used to create new GnomeGeneric object instances.
- * @data: The closure data to be passed to the @factory callback routine.
+ * @user_data: The closure data to be passed to the @factory callback routine.
  *
  * Initializes @c_factory with the command-line arguments and registers
  * the new factory in the name server.
@@ -98,12 +99,12 @@ create_bonobo_generic_factory (BonoboObject *object)
  * Returns: The initialized BonoboGenericFactory object.
  */
 BonoboGenericFactory *
-bonobo_generic_factory_construct (const char             *goad_id,
+bonobo_generic_factory_construct (const char             *oaf_iid,
 				  BonoboGenericFactory   *c_factory,
 				  CORBA_Object            corba_factory,
 				  BonoboGenericFactoryFn  factory,
 				  GnomeFactoryCallback    factory_cb,
-				  void                   *data)
+				  gpointer                user_data)
 {
 	CORBA_Environment ev;
 	int ret;
@@ -114,14 +115,16 @@ bonobo_generic_factory_construct (const char             *goad_id,
 
 	bonobo_object_construct (BONOBO_OBJECT (c_factory), corba_factory);
 
+	bonobo_running_context_ignore_object (corba_factory);
+
 	c_factory->factory         = factory;
 	c_factory->factory_cb      = factory_cb;
-	c_factory->factory_closure = data;
-	c_factory->goad_id         = g_strdup (goad_id);
+	c_factory->factory_closure = user_data;
+	c_factory->oaf_iid         = g_strdup (oaf_iid);
 
 	CORBA_exception_init (&ev);
 
-	ret = oaf_active_server_register (c_factory->goad_id, corba_factory);
+	ret = oaf_active_server_register (c_factory->oaf_iid, corba_factory);
 
 	CORBA_exception_free (&ev);
 
@@ -135,9 +138,9 @@ bonobo_generic_factory_construct (const char             *goad_id,
 
 /**
  * bonobo_generic_factory_new:
- * @goad_id: The GOAD id that this factory implements
+ * @oaf_iid: The GOAD id that this factory implements
  * @factory: A callback which is used to create new BonoboObject instances.
- * @data: The closure data to be passed to the @factory callback routine.
+ * @user_data: The closure data to be passed to the @factory callback routine.
  *
  * This is a helper routine that simplifies the creation of factory
  * objects for GNOME objects.  The @factory function will be
@@ -151,9 +154,9 @@ bonobo_generic_factory_construct (const char             *goad_id,
  * name server.
  */
 BonoboGenericFactory *
-bonobo_generic_factory_new (const char             *goad_id,
+bonobo_generic_factory_new (const char             *oaf_iid,
 			    BonoboGenericFactoryFn  factory,
-			    void                   *data)
+			    gpointer                user_data)
 {
 	BonoboGenericFactory *c_factory;
 	GNOME_ObjectFactory corba_factory;
@@ -162,19 +165,19 @@ bonobo_generic_factory_new (const char             *goad_id,
 	
 	c_factory = gtk_type_new (bonobo_generic_factory_get_type ());
 
-	corba_factory = create_bonobo_generic_factory (BONOBO_OBJECT (c_factory));
+	corba_factory = bonobo_generic_factory_corba_object_create (BONOBO_OBJECT (c_factory));
 	if (corba_factory == CORBA_OBJECT_NIL) {
 		bonobo_object_unref (BONOBO_OBJECT (c_factory));
 		return NULL;
 	}
 	
 	return bonobo_generic_factory_construct (
-		goad_id, c_factory, corba_factory, factory, NULL, data);
+		oaf_iid, c_factory, corba_factory, factory, NULL, user_data);
 }
 
 /**
  * bonobo_generic_factory_new:
- * @goad_id: The GOAD id that this factory implements
+ * @oaf_iid: The GOAD id that this factory implements
  * @factory_cb: A callback which is used to create new BonoboObject instances.
  * @data: The closure data to be passed to the @factory callback routine.
  *
@@ -190,7 +193,7 @@ bonobo_generic_factory_new (const char             *goad_id,
  * name server.
  */
 BonoboGenericFactory *bonobo_generic_factory_new_multi (
-	const char *goad_id,
+	const char *oaf_iid,
 	GnomeFactoryCallback factory_cb,
 	gpointer data)
 {
@@ -198,18 +201,18 @@ BonoboGenericFactory *bonobo_generic_factory_new_multi (
 	GNOME_ObjectFactory corba_factory;
 
 	g_return_val_if_fail (factory_cb != NULL, NULL);
-	g_return_val_if_fail (goad_id != NULL, NULL);
+	g_return_val_if_fail (oaf_iid != NULL, NULL);
 	
 	c_factory = gtk_type_new (bonobo_generic_factory_get_type ());
 
-	corba_factory = create_bonobo_generic_factory (BONOBO_OBJECT (c_factory));
+	corba_factory = bonobo_generic_factory_corba_object_create (BONOBO_OBJECT (c_factory));
 	if (corba_factory == CORBA_OBJECT_NIL) {
 		bonobo_object_unref (BONOBO_OBJECT (c_factory));
 		return NULL;
 	}
 	
 	return bonobo_generic_factory_construct (
-		goad_id, c_factory, corba_factory, NULL, factory_cb, data);
+		oaf_iid, c_factory, corba_factory, NULL, factory_cb, data);
 }
 
 
@@ -220,25 +223,26 @@ bonobo_generic_factory_finalize (GtkObject *object)
 	CORBA_Environment ev;
 
 	CORBA_exception_init (&ev);
-	oaf_active_server_unregister (c_factory->goad_id,
+	oaf_active_server_unregister (c_factory->oaf_iid,
 				      BONOBO_OBJECT (c_factory)->corba_objref);
 	CORBA_exception_free (&ev);
-	g_free (c_factory->goad_id);
+	g_free (c_factory->oaf_iid);
 	
-	GTK_OBJECT_CLASS (bonobo_generic_factory_parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (bonobo_generic_factory_parent_class)->finalize (object);
 }
 
 static BonoboObject *
 bonobo_generic_factory_new_generic (BonoboGenericFactory *factory,
-				    const char           *goad_id)
+				    const char           *oaf_iid)
 {
 	g_return_val_if_fail (factory != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_GENERIC_FACTORY (factory), NULL);
 
-	if(factory->factory_cb)
-		return (*factory->factory_cb)(factory, goad_id, factory->factory_closure);
+	if (factory->factory_cb)
+		return factory->factory_cb (factory, oaf_iid,
+					    factory->factory_closure);
 	else
-		return (*factory->factory)(factory, factory->factory_closure);
+		return factory->factory    (factory, factory->factory_closure);
 }
 
 static void
@@ -261,11 +265,6 @@ bonobo_generic_factory_class_init (BonoboGenericFactoryClass *klass)
 	init_generic_factory_corba_class ();
 }
 
-static void
-bonobo_generic_factory_init (BonoboObject *object)
-{
-}
-
 /**
  * bonobo_generic_factory_get_type:
  *
@@ -282,7 +281,7 @@ bonobo_generic_factory_get_type (void)
 			sizeof (BonoboGenericFactory),
 			sizeof (BonoboGenericFactoryClass),
 			(GtkClassInitFunc) bonobo_generic_factory_class_init,
-			(GtkObjectInitFunc) bonobo_generic_factory_init,
+			(GtkObjectInitFunc) NULL,
 			NULL, /* reserved 1 */
 			NULL, /* reserved 2 */
 			(GtkClassInitFunc) NULL
