@@ -71,14 +71,8 @@ bonobo_moniker_set_parent (BonoboMoniker     *moniker,
 {
 	g_return_if_fail (BONOBO_IS_MONIKER (moniker));
 	
-	if (moniker->priv->parent != CORBA_OBJECT_NIL)
-		bonobo_object_release_unref (moniker->priv->parent, ev);
-
-	if (parent != CORBA_OBJECT_NIL)
-		moniker->priv->parent =
-			bonobo_object_dup_ref (parent, ev);
-	else
-		moniker->priv->parent = CORBA_OBJECT_NIL;
+	bonobo_object_release_unref (moniker->priv->parent, ev);
+	moniker->priv->parent = bonobo_object_dup_ref (parent, ev);
 }
 
 /**
@@ -115,13 +109,16 @@ bonobo_moniker_get_parent (BonoboMoniker     *moniker,
 const char *
 bonobo_moniker_get_name (BonoboMoniker *moniker)
 {	
+	const char *str;
+
 	g_return_val_if_fail (BONOBO_IS_MONIKER (moniker), "");
 
-	if (moniker->priv->name)
-		return & moniker->priv->name [
-			moniker->priv->prefix_len];
+	str = CLASS (moniker)->get_name (moniker);
 
-	return "";
+	if (str)
+		return str + moniker->priv->prefix_len;
+	else
+		return "";
 }
 
 /**
@@ -138,10 +135,7 @@ bonobo_moniker_get_name_full (BonoboMoniker *moniker)
 {	
 	g_return_val_if_fail (BONOBO_IS_MONIKER (moniker), "");
 
-	if (moniker->priv->name)
-		return moniker->priv->name;
-
-	return "";
+	return CLASS (moniker)->get_name (moniker);
 }
 
 /**
@@ -156,7 +150,10 @@ bonobo_moniker_get_name_full (BonoboMoniker *moniker)
 char *
 bonobo_moniker_get_name_escaped (BonoboMoniker *moniker)
 {
-	return bonobo_moniker_util_escape (moniker->priv->name, 0);
+	g_return_val_if_fail (BONOBO_IS_MONIKER (moniker), "");
+
+	return bonobo_moniker_util_escape (
+		CLASS (moniker)->get_name (moniker), 0);
 }
 
 /**
@@ -172,12 +169,49 @@ bonobo_moniker_set_name (BonoboMoniker *moniker,
 			 const char    *name,
 			 int            num_chars)
 {
+	char *str;
+
 	g_return_if_fail (BONOBO_IS_MONIKER (moniker));
-	g_return_if_fail (strlen (name) >= moniker->priv->prefix_len);
+
+	str = bonobo_moniker_util_unescape (name, num_chars);
+
+	CLASS (moniker)->set_name (moniker, str);
+
+	g_free (str);
+}
+
+/**
+ * bonobo_moniker_get_prefix:
+ * @moniker: a moniker
+ * 
+ * Return value: the registered prefix for this moniker or
+ * NULL if there isn't one. eg "file:"
+ **/
+const char *
+bonobo_moniker_get_prefix (BonoboMoniker *moniker)
+{
+	g_return_val_if_fail (BONOBO_IS_MONIKER (moniker), "");
+
+	return moniker->priv->prefix;
+}
+
+static void
+impl_bonobo_moniker_set_name (BonoboMoniker *moniker,
+			      const char    *unescaped_name)
+{
+	g_return_if_fail (BONOBO_IS_MONIKER (moniker));
+	g_return_if_fail (strlen (unescaped_name) >= moniker->priv->prefix_len);
 
 	g_free (moniker->priv->name);
-	moniker->priv->name = bonobo_moniker_util_unescape (
-		name, num_chars);
+	moniker->priv->name = g_strdup (unescaped_name);
+}
+
+static const char *
+impl_bonobo_moniker_get_name (BonoboMoniker *moniker)
+{
+	g_return_val_if_fail (BONOBO_IS_MONIKER (moniker), "");
+
+	return moniker->priv->name;
 }
 
 static CORBA_char *
@@ -188,7 +222,7 @@ bonobo_moniker_default_get_display_name (BonoboMoniker     *moniker,
 	char       *tmp;
 	
 	parent_name = bonobo_moniker_util_get_parent_name (
-		bonobo_object_corba_objref (BONOBO_OBJECT (moniker)), ev);
+		BONOBO_OBJREF (moniker), ev);
 
 	if (BONOBO_EX (ev))
 		return NULL;
@@ -221,7 +255,6 @@ bonobo_moniker_default_parse_display_name (BonoboMoniker     *moniker,
 	
 	g_return_val_if_fail (moniker != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (moniker->priv != NULL, CORBA_OBJECT_NIL);
-	g_return_val_if_fail (moniker->priv->prefix != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (strlen (name) >= moniker->priv->prefix_len, CORBA_OBJECT_NIL);
 
 	bonobo_moniker_set_parent (moniker, parent, ev);
@@ -230,9 +263,8 @@ bonobo_moniker_default_parse_display_name (BonoboMoniker     *moniker,
 
 	bonobo_moniker_set_name (moniker, name, i);
 
-	return bonobo_moniker_util_new_from_name_full (
-		bonobo_object_corba_objref (BONOBO_OBJECT (moniker)),
-		&name [i], ev);	
+	return bonobo_moniker_util_new_from_name_full (BONOBO_OBJREF (moniker),
+						       &name [i], ev);	
 }
 
 static CORBA_char *
@@ -269,7 +301,9 @@ impl_resolve (PortableServer_Servant       servant,
 					   requested_interface, ev);
 
 	/* Try an extender */
-	if (!BONOBO_EX (ev) && retval == CORBA_OBJECT_NIL) {
+	if (!BONOBO_EX (ev) && retval == CORBA_OBJECT_NIL &&
+	    moniker->priv->prefix) {
+
 		Bonobo_Unknown extender;
 		
 		extender = bonobo_moniker_find_extender (
@@ -281,8 +315,7 @@ impl_resolve (PortableServer_Servant       servant,
 
 		else if (extender != CORBA_OBJECT_NIL) {
 			retval = Bonobo_MonikerExtender_resolve (
-				extender,
-				bonobo_object_corba_objref (BONOBO_OBJECT (moniker)),
+				extender, BONOBO_OBJREF (moniker),
 				options, moniker->priv->name,
 				requested_interface, ev);
 
@@ -367,6 +400,9 @@ bonobo_moniker_class_init (BonoboMonikerClass *klass)
 	klass->get_display_name = bonobo_moniker_default_get_display_name;
 	klass->parse_display_name = bonobo_moniker_default_parse_display_name;
 
+	klass->set_name   = impl_bonobo_moniker_set_name;
+	klass->get_name   = impl_bonobo_moniker_get_name;
+
 	init_moniker_corba_class ();
 }
 
@@ -436,7 +472,7 @@ bonobo_moniker_corba_object_create (BonoboObject *object)
  * @moniker: an un-constructed moniker object.
  * @corba_moniker: a CORBA handle inheriting from Bonobo::Moniker, or
  * CORBA_OBJECT_NIL, in which case a base Bonobo::Moniker is created.
- * @prefix: the prefix name of the moniker eg. 'file:', '!' or 'tar:'
+ * @prefix: the prefix name of the moniker eg. 'file:', '!' or 'tar:' or NULL
  * 
  *  Constructs a newly created bonobo moniker with the given arguments.
  * 
