@@ -70,9 +70,21 @@ static GtkObjectClass *bonobo_object_parent_class;
 
 #ifdef BONOBO_REF_HOOKS
 
-static int         ref_indent = 0;
 static GHashTable *living_ao_ht = NULL;
 
+static void
+bonobo_debug_print (char *name, char *fmt, ...)
+{
+	va_list args;
+           
+	va_start (args, fmt);
+	
+	printf ("[%06d]:%-15s ", getpid (), name); 
+	vprintf (fmt, args);
+	printf ("\n"); 
+
+	va_end (args);
+}
 #endif /* BONOBO_REF_HOOKS */
 
 /**
@@ -173,6 +185,13 @@ bonobo_object_finalize (BonoboAggregateObject *ao)
 			g_assert (BONOBO_OBJECT (o)->priv->ao != NULL);
 #ifdef BONOBO_REF_HOOKS
 			g_assert (BONOBO_OBJECT (o)->priv->ao->destroyed);
+
+			bonobo_debug_print ("finalize", 
+					    "[%p] %-20s corba_objref=[%p]"
+					    " gtk_ref_count=%d", o,
+					    gtk_type_name (GTK_OBJECT_TYPE (o)),
+					    BONOBO_OBJECT (o)->corba_objref,
+					    GTK_OBJECT (o)->ref_count);
 #endif
 
 			/*
@@ -262,7 +281,6 @@ bonobo_object_trace_refs (BonoboObject *object,
 			  gboolean      ref)
 {
 #ifdef BONOBO_REF_HOOKS
-	char *indent;
 	BonoboAggregateObject *ao;
 	BonoboDebugRefData *descr;
 	
@@ -281,21 +299,19 @@ bonobo_object_trace_refs (BonoboObject *object,
 		
 		object->priv->ao->ref_count++;
 		
-		indent = g_strnfill (++ref_indent, ' ');
-		g_printerr ("%s[%d]:Ref %s:[%p] to %d at %s:%d\n", indent, getpid(),
-			    gtk_type_name (GTK_OBJECT (object)->klass->type),
-			    object, ao->ref_count, fn, line);
+		bonobo_debug_print ("ref", "[%p]:[%p]:%s to %d at %s:%d", 
+			object, ao,
+		        gtk_type_name (GTK_OBJECT (object)->klass->type),
+			ao->ref_count, fn, line);
 
-		g_free (indent);
 	} else { /* unref */
-		indent = g_strnfill (ref_indent--, ' ');
-		g_printerr ("%s[%d]:UnRef %s:[%p] from %d at %s:%d\n", indent, getpid(),
-			    gtk_type_name (GTK_OBJECT (object)->klass->type),
-			    ao, ao->ref_count, fn, line);
-		g_free (indent);
+		bonobo_debug_print ("unref", "[%p]:[%p]:%s from %d at %s:%d", 
+			object, ao,
+			gtk_type_name (GTK_OBJECT (object)->klass->type),
+			ao->ref_count, fn, line);
 
 		g_return_if_fail (ao->ref_count > 0);
-	
+
 		if (ao->ref_count == 1) {
 			bonobo_object_destroy (ao);
 
@@ -318,10 +334,8 @@ bonobo_object_trace_refs (BonoboObject *object,
 			
 			bonobo_object_finalize (ao);
 		} else if (ao->ref_count < 0) {
-			indent = g_strnfill (ref_indent + 1, ' ');
-			g_printerr ("%sUnusual: [%p] already finalized\n",
-				    indent, ao);
-			g_free (indent);
+			bonobo_debug_print ("unusual", 
+					    "[%p] already finalized", ao);
 		}
 	}
 #else
@@ -531,8 +545,8 @@ bonobo_object_query_local_interface (BonoboObject *object,
 
 static CORBA_Object
 impl_Bonobo_Unknown_queryInterface (PortableServer_Servant  servant,
-				     const CORBA_char       *repoid,
-				     CORBA_Environment      *ev)
+				    const CORBA_char       *repoid,
+				    CORBA_Environment      *ev)
 {
 	BonoboObject *object = bonobo_object_from_servant (servant);
 	BonoboObject *local_interface;
@@ -540,6 +554,14 @@ impl_Bonobo_Unknown_queryInterface (PortableServer_Servant  servant,
 	local_interface = bonobo_object_query_local_interface (
 		object, repoid);
 
+#ifdef BONOBO_REF_HOOKS
+	bonobo_debug_print ("query-interface", 
+			    "[%p]:[%p]:%s repoid=%s", 
+			    object, object->priv->ao,
+			    gtk_type_name (GTK_OBJECT (object)->klass->type),
+			    repoid);
+#endif
+
 	if (local_interface == NULL)
 		return CORBA_OBJECT_NIL;
 
@@ -670,11 +692,9 @@ bonobo_object_instance_init (GtkObject    *gtk_object,
 
 #ifdef BONOBO_REF_HOOKS
 	{
-		char *indent = g_strnfill (++ref_indent, ' ');
-		g_printerr ("%s[%d]:Create %s:[%p] to %d\n", indent, getpid(),
-			    gtk_type_name (klass->type),
-			    ao, ao->ref_count);
-		g_free (indent);
+		bonobo_debug_print ("create", "[%p]:[%p]:%s to %d", object, ao,
+				    gtk_type_name (klass->type),
+				    ao->ref_count);
 
 		g_assert (g_hash_table_lookup (living_ao_ht, ao) == NULL);
 		g_hash_table_insert (living_ao_ht, ao, ao);
@@ -723,24 +743,32 @@ bonobo_ao_debug_foreach (gpointer key, gpointer value, gpointer user_data)
 
 	g_return_if_fail (ao != NULL);
 
-	g_print ("[%p]\tref_count=%d, %d interfaces:\n", ao,
-		   ao->ref_count, g_list_length (ao->objs));
+	bonobo_debug_print ("object-status", 
+			    "[%p] %-20s ref_count=%d, interfaces=%d", ao, "",
+			    ao->ref_count, g_list_length (ao->objs));
 		
 	for (l = ao->objs; l; l = l->next) {
 		BonoboObject *object = BONOBO_OBJECT (l->data);
-
-		g_print ("- [%p] %20s\tgtk_ref_count=%d\n", object,
-			   gtk_type_name (GTK_OBJECT_TYPE (object)),
-			   GTK_OBJECT (object)->ref_count);
+		
+		bonobo_debug_print ("", "[%p] %-20s corba_objref=[%p]"
+				    " gtk_ref_count=%d", object,
+				    gtk_type_name (GTK_OBJECT_TYPE (object)),
+				    object->corba_objref,
+				    GTK_OBJECT (object)->ref_count);
 	}
-	g_print ("Referencing: \n");
-	for (l = g_list_last (ao->refs); l; l = l->prev) {
+
+	l = g_list_last (ao->refs);
+
+	if (l)
+		bonobo_debug_print ("referencing" ,"");
+
+	for (; l; l = l->prev) {
 		BonoboDebugRefData *descr = l->data;
 
-		g_print ("%s -\t%s:%d\n", descr->ref?"ref":"unref",
-			 descr->fn, descr->line);
+		bonobo_debug_print ("", "%-7s - %s:%d", 
+				    descr->ref ? "ref" : "unref",
+				    descr->fn, descr->line);
 	}
-	g_print ("\n");
 }
 #endif
 
@@ -748,17 +776,20 @@ static void
 bonobo_object_shutdown (void)
 {
 #ifdef BONOBO_REF_HOOKS
+	
+	bonobo_debug_print ("shutdown-start", 
+		"-------------------------------------------------");
 
 	if (living_ao_ht)
 		g_hash_table_foreach (living_ao_ht,
 				      bonobo_ao_debug_foreach, NULL);
 
-	if (g_hash_table_size (living_ao_ht) > 0)
-		g_print ("living bonobo objects count = %d\n",
-			   g_hash_table_size (living_ao_ht));
-	else
-		g_print ("No object references leaked\n");
+	bonobo_debug_print ("living-objects",
+			    "living bonobo objects count = %d",
+			    g_hash_table_size (living_ao_ht));
 
+	bonobo_debug_print ("shutdown-end", 
+		"-------------------------------------------------");
 #endif
 }
 
@@ -791,13 +822,13 @@ bonobo_object_activate_servant (BonoboObject *object, void *servant)
 	CORBA_exception_init (&ev);
 
 	CORBA_free (PortableServer_POA_activate_object (
-		bonobo_poa (), servant, &ev));
+                bonobo_poa (), servant, &ev));
 
 	o = PortableServer_POA_servant_to_reference (
 		bonobo_poa(), servant, &ev);
 
 	CORBA_exception_free (&ev);
-
+	
 	if (o) {
 		object->corba_objref = o;
 		bonobo_object_bind_to_servant (object, servant);
@@ -850,6 +881,9 @@ bonobo_object_add_interface (BonoboObject *object, BonoboObject *newobj)
        BonoboAggregateObject *oldao, *ao;
        GList *l;
 
+       g_return_if_fail (object->priv->ao->ref_count > 0);
+       g_return_if_fail (newobj->priv->ao->ref_count > 0);
+
        if (object->priv->ao == newobj->priv->ao)
                return;
 
@@ -867,6 +901,17 @@ bonobo_object_add_interface (BonoboObject *object, BonoboObject *newobj)
 
        ao = object->priv->ao;
        oldao = newobj->priv->ao;
+       ao->ref_count = ao->ref_count + oldao->ref_count - 1;
+
+#ifdef BONOBO_REF_HOOKS		
+       bonobo_debug_print ("add_interface", 
+			   "[%p]:[%p]:%s to [%p]:[%p]:%s ref_count=%d", 
+			   object, object->priv->ao,
+			   gtk_type_name (GTK_OBJECT (object)->klass->type),
+			   newobj, newobj->priv->ao,
+			   gtk_type_name (GTK_OBJECT (newobj)->klass->type),
+			   ao->ref_count);
+#endif
 
        /* Merge the two AggregateObject lists */
        for (l = oldao->objs; l; l = l->next) {
@@ -886,6 +931,7 @@ bonobo_object_add_interface (BonoboObject *object, BonoboObject *newobj)
 #ifdef BONOBO_REF_HOOKS
        g_assert (g_hash_table_lookup (living_ao_ht, oldao) == oldao);
        g_hash_table_remove (living_ao_ht, oldao);
+       ao->refs = g_list_concat (ao->refs, oldao->refs);
 #endif
 
        g_list_free (oldao->objs);
