@@ -8,6 +8,7 @@
  */
 #include <config.h>
 #include <stdarg.h>
+#include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-print-client.h>
 #include <libgnomeprint/gnome-print-meta.h>
 #include <libart_lgpl/art_affine.h>
@@ -59,7 +60,7 @@ bonobo_print_client_render (BonoboPrintClient   *client,
 		g_warning ("Component print returns no data");
 		return;
 	}
-	if (ev._major != CORBA_NO_EXCEPTION) {
+	if (BONOBO_EX (&ev)) {
 		g_warning ("Component print exception");
 		return;
 	}
@@ -222,7 +223,7 @@ bonobo_print_client_get (BonoboObjectClient *object)
 
 	interf = bonobo_object_client_query_interface (object, "IDL:Bonobo/Print:1.0", &ev);
 
-	if (ev._major != CORBA_NO_EXCEPTION)
+	if (BONOBO_EX (&ev))
 		g_warning ("Exception getting print interface");
 	else if (interf == CORBA_OBJECT_NIL)
 		g_warning ("No printing interface");
