@@ -1,4 +1,4 @@
-/* $Id: component.c,v 1.17 2000/12/11 23:11:42 michael Exp $ */
+/* $Id: component.c,v 1.19 2001/01/18 10:30:15 dietmar Exp $ */
 
 #include "component.h"
 #include "embeddable-io.h"
@@ -119,9 +119,8 @@ sample_client_site_add_frame (SampleClientSite *site)
 	 * so that it can merge menu and toolbar items when it gets
 	 * activated.
 	 */
-	view_frame = bonobo_client_site_new_view (
-		BONOBO_CLIENT_SITE (site),
-		bonobo_object_corba_objref (BONOBO_OBJECT (site->app->ui_container)));
+	view_frame = bonobo_client_site_new_view (BONOBO_CLIENT_SITE (site),
+		BONOBO_OBJREF (site->app->ui_container));
 	
 	/*
 	 * Embed the view frame into the application.
@@ -263,10 +262,7 @@ load_stream_cb (GtkWidget *caller, SampleClientSite *site)
 		/*
 		 * Load the file into the component using PersistStream.
 		 */
-		Bonobo_PersistStream_load (persist,
-					   (Bonobo_Stream)
-					   bonobo_object_corba_objref
-					   (BONOBO_OBJECT (stream)),
+		Bonobo_PersistStream_load (persist, BONOBO_OBJREF (stream),
 					   "", &ev);
 
 		if (ev._major != CORBA_NO_EXCEPTION) {
@@ -435,7 +431,6 @@ sample_client_site_new (BonoboItemContainer *container,
 		bonobo_client_site_bind_embeddable (BONOBO_CLIENT_SITE (site),
 						    embeddable);
 		bonobo_object_unref (BONOBO_OBJECT (embeddable));
-		bonobo_item_container_add (container, BONOBO_OBJECT (site));
 
 		site->app = app;
 		g_free (site->obj_id);
